/* BEGIN_ICS_COPYRIGHT2 ****************************************

Copyright (c) 2015, 2017, Intel Corporation

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of Intel Corporation nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

NOTICE

This software is provided as “pre-release” software for the purpose of 
your testing and evaluation to enable your qualification schedules to
align closely with the final Intel public release. Pre-release means the 
software which has not completed final validation, may contain bugs or
functional deficiencies, and may be modified by or for Intel prior to
final release.  The provided software is not intended for public 
disclosure, use in a production environment, or use by any end customer
or anyone other than your internal software evaluation personnel.  You
understand, acknowledge and agree that: (i) pre-release software may not
be fully tested and may contain bugs, errors or functional deficiencies;
(ii) pre-release software is not suitable for commercial release in its
current state; (iii) regulatory approvals for pre-release software (such
as UL or FCC) have not been obtained, and pre-release software may
therefore not be certified for use in certain countries or environments
and (iv) Intel can provide no assurance that it will ever produce or 
make generally available a production version of the pre-release
software. Intel is not under any obligation to develop and/or release or 
offer for sale or license a final product based upon the pre-release
software and may unilaterally elect to abandon the pre-release software
or any such development platform at any time and without any obligation
or liability whatsoever to You or any other person.  Intel Corporation
requests that you do not release, publish, or distribute this
pre-release software until you are specifically authorized. These terms
are deleted upon publication of this software.

 * ** END_ICS_COPYRIGHT2   ****************************************/

#ifdef __LINUX__
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/stat.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include "hsm_com_srvr_api.h"
#include "hsm_com_srvr_data.h"


	
int
validate_header(char *buf, int len)
{
	hsm_com_common_t	*common;
	int					expect;


	if(len < sizeof(hsm_com_common_t)){
		return(-1);
	}

	common = (hsm_com_common_t*)buf;
	expect = common->payload_len + sizeof(hsm_com_common_t);


	if(expect == len){
		return(0);
	}else if(expect < len){
		return(expect); // expect is the offset into the buffer where the next packet starts.
	}else{
		return(-1); // Didn't get the whole thing yet.
	}



	
}


int
unix_sck_read_data(int fd, hsm_com_scratch_t *scratch, char *buf, int buf_len, int *offset)
{
	char	*temp = NULL;
	int		len = 0;
	int		nread;
	int		next_packet = 0;
	char    *data_ptr;

	*offset = 0;

	if((nread = read(fd, buf, buf_len)) <= 0){
		// Error on socket...
		return(0);
	}
	//printf("Got: %d bytes + prefill of %d:%d\n",nread,scratch->scratch_fill,scratch->scratch_len);

	if(scratch->scratch_fill){
		// Malloc space for fill + new data...
		if((temp = malloc(scratch->scratch_fill + nread)) == NULL){
			// Error..
			//printf("Could not allocate temp buffer\n");
			if (temp)
				free(temp);
			return(0);
		}

		//printf("Got: %d bytes + prefill of %d\n",nread,scratch->scratch_fill);
		memcpy(&temp[0],scratch->scratch,scratch->scratch_fill);
		memcpy(&temp[scratch->scratch_fill],buf,nread);
		data_ptr = temp;
		len = nread + scratch->scratch_fill;

	}else{
		data_ptr = buf;
		len = nread;
	}
	

	while((next_packet = validate_header(data_ptr,len)) > 0){
		if(next_packet > len){
			// Error 
			//printf("unix_sck_read_data: mis-formatted data. packet len too big\n");
			if (temp) free(temp);
			return(-1);
		}
		
		*offset += next_packet;
		data_ptr = &buf[*offset];
		len -= next_packet;
	
	}

	if(next_packet == 0){
		
		if(temp){
			memcpy(buf,data_ptr,len);
			free(temp);
			*offset = 0;
			scratch->scratch_fill = 0;
		}

		return len;
	}else{
		if(len <= scratch->scratch_len){
			memcpy(scratch->scratch,data_ptr,len);
			scratch->scratch_fill = len;
		}
		if(temp)
			free(temp);
		return(-2);
	}

	if(temp)
		free(temp);

	return (-1);


}
#endif
